/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.pool.impl.GenericObjectPool;

import repast.simphony.freezedry.datasource.DBConnectionInfo;

/**
 * repast.simphony.data.db
 *
 * @author Sascha Holzhauer
 * @date 21.03.2011 
 *
 */
public class CustomPoolingDataSource extends PoolingDataSource {
	
	/**
	 * @param connectionPool
	 */
	public CustomPoolingDataSource(GenericObjectPool connectionPool) {
		super(connectionPool);
	}
	
	/**
	 * 
	 */
	public void clearPool() {
		try {
			this._pool.clear();
		} catch (UnsupportedOperationException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
	/**
	 * @param info
	 * @return pooling data source
	 * @throws ClassNotFoundException
	 */
	public static CustomPoolingDataSource getDataSource(DBConnectionInfo info) throws ClassNotFoundException {
		if (info.driverName != null) {
			Class.forName(info.driverName);
		}

		GenericObjectPool connectionPool = new GenericObjectPool(null);
		connectionPool.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_GROW);
		ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(info.dbURL,
				info.userName, info.password);
		@SuppressWarnings("unused")
		PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(
				connectionFactory, connectionPool, null, null, false, true);
		CustomPoolingDataSource dataSource = new CustomPoolingDataSource(connectionPool);

		return dataSource;
	}
}
