/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.engine;


import repast.simphony.data.db.outputter.DefaultDBOutputter;
import repast.simphony.data.db.outputter.gui.DbOutputterMenuItem;
import repast.simphony.data2.DataConstants;
import repast.simphony.data2.DataSetManager;
import repast.simphony.data2.DataSetRegistry;
import repast.simphony.data2.DataSink;
import repast.simphony.data2.builder.DataSetBuilder;
import repast.simphony.engine.controller.DescriptorControllerAction;
import repast.simphony.engine.environment.ControllerAction;
import repast.simphony.engine.environment.DefaultControllerAction;
import repast.simphony.engine.environment.RunState;
import simphony.util.messages.MessageCenter;


/**
 * repast.simphony.data.db
 * 
 * The {@link ControllerAction} that is registered and added to the scenario tree (by
 * {@link DbOutputterMenuItem}. Passes the {@link OutputterDescriptor} to super constructor.
 * 
 * The class is necessary to distinguish between DbOutputter and FileOutputter, at least when it comes to
 * ActionEditorCreators.
 * 
 * @author Sascha Holzhauer
 * @param <T> Type of Outputter
 * @param <U> Type of OutputterDescriptor
 * @date 14.04.2010
 * 
 */
public class DbOutputterComponentControllerAction<U extends DbOutputterDescriptor> extends
		DefaultControllerAction implements DescriptorControllerAction<DbOutputterDescriptor> {

	protected U outputterDescriptor;
	protected DataSink outputter;
	
	/**
	 * Logger
	 */
	private static final MessageCenter	LOG	= MessageCenter.getMessageCenter(DefaultDBOutputter.class);

	protected DbOutputterDescriptor descriptor;
	
	 private static MessageCenter msgCenter = MessageCenter
      .getMessageCenter(DbOutputterComponentControllerAction.class);
	  
	/**
	 * @param outputterDescriptor
	 */
	public DbOutputterComponentControllerAction(U descriptor) {
		this. descriptor = descriptor;
	}

	public DataSink getOutputter() {
		if (outputter == null) {
			outputter = new DefaultDBOutputter(this.descriptor);
		}
		return outputter;
	}
	
	
	/**
	 * @see repast.simphony.data.logging.outputter.engine.DefaultOutputterDescriptorAction#runCleanup(repast.simphony.engine.environment.RunState,
	 *      java.lang.Object)
	 */
	@Override
	public void runCleanup(RunState runState, Object contextId) {
		getOutputter().close();
		super.runCleanup(runState, contextId);
		LOG.debug("DbOutputterDescriptorAction (" + this.getDescriptor().getName() + "): runCleanup invoked");
	}

	  /*
	   * (non-Javadoc)
	   * 
	   * @see
	   * repast.simphony.engine.environment.DefaultControllerAction#batchInitialize
	   * (repast.simphony.engine.environment.RunState, java.lang.Object)
	   */
	  @Override
	  public void batchInitialize(RunState runState, Object contextId) {
	    // in batch this is called before the whole batch of runs
	    // in non-batch this is called before each run
	    DataSetRegistry registry = (DataSetRegistry)runState.getFromRegistry(DataConstants.REGISTRY_KEY);
	    DataSetManager manager = registry.getDataSetManager(contextId);
	    DataSetBuilder<?> builder = manager.getDataSetBuilder(descriptor.getDataSet());

	    if (builder == null) {
	      msgCenter.error("Error while creating DbOutputter. DataSet '" + descriptor.getDataSet()
	          + "' not found", new NullPointerException());
	    } else {

	         builder.addDataSink(getOutputter());
	    }
	  }
	  
	@Override
	public DbOutputterDescriptor getDescriptor() {
		return descriptor;
	}
}
